/* messaging functions
   Copyright 2003-2005	Craig Franklin
 */

#include "stdhdr.h"
#include "libgputils.h"
#include <stdarg.h>

bool gp_quiet           = false;
bool gp_message_disable = false;
bool gp_debug_disable   = true;

int gp_num_errors   = 0;
int gp_num_warnings = 0;
int gp_num_messages = 0;

static int _real_num_errors   = 0;
static int _real_num_warnings = 0;
static int _real_num_messages = 0;

/*------------------------------------------------------------------------------------------------*/

FUNC(void) gp_error(const char* Format, ...) {
  va_list args;
  char    buffer[BUFSIZ];

  _real_num_errors++;

  if (gp_message_disable) {
    return;
  }

  gp_num_errors++;

  if (gp_quiet) {
    return;
  }

  va_start(args, Format);
  vsnprintf(buffer, sizeof(buffer), Format, args);
  va_end(args);

  printf("error: %s\n", buffer);
}

/*------------------------------------------------------------------------------------------------*/

FUNC(void) gp_warning(const char* Format, ...)
{
  va_list args;
  char    buffer[BUFSIZ];

  _real_num_warnings++;

  if (gp_message_disable) {
    return;
  }

  gp_num_warnings++;

  if (gp_quiet) {
    return;
  }

  va_start(args, Format);
  vsnprintf(buffer, sizeof(buffer), Format, args);
  va_end(args);

  printf("warning: %s\n", buffer);
}

/*------------------------------------------------------------------------------------------------*/

FUNC(void) gp_message(const char* Format, ...)
{
  va_list args;
  char    buffer[BUFSIZ];

  _real_num_messages++;

  if (gp_message_disable) {
    return;
  }

  gp_num_messages++;

  if (gp_quiet) {
    return;
  }

  va_start(args, Format);
  vsnprintf(buffer, sizeof(buffer), Format, args);
  va_end(args);

  printf("message: %s\n", buffer);
}

/*------------------------------------------------------------------------------------------------*/

FUNC(void) gp_debug(const char* Format, ...) {
  va_list args;
  char    buffer[BUFSIZ];

  if (gp_debug_disable) {
    return;
  }

  if (gp_quiet) {
    return;
  }

  va_start(args, Format);
  vsnprintf(buffer, sizeof(buffer), Format, args);
  va_end(args);

  printf("debug: %s\n", buffer);
}

/*------------------------------------------------------------------------------------------------*/

int gp_real_num_errors(void)
{
  return _real_num_errors;
}

/*------------------------------------------------------------------------------------------------*/

int
gp_real_num_warnings(void)
{
  return _real_num_warnings;
}

/*------------------------------------------------------------------------------------------------*/

int
gp_real_num_messages(void)
{
  return _real_num_messages;
}
