/* General system functions
   Copyright 2003, 2004, 2005
   Craig Franklin

    Copyright 2015-2016 Molnár Károly
*/

#pragma once

/* Used in disasm. */
#define EXPLANATION_DISTANCE            60
#define TABULATOR_SIZE                  8
#define ARRAY_SIZE(a)                   (sizeof(a) / sizeof((a)[0]))

#define FlagSet(v, f)                   ((v) |= (f))
#define FlagClr(v, f)                   ((v) &= ~(f))
#define FlagIsSet(v, f)                 (((v) & (f)) != 0)
#define FlagIsClr(v, f)                 (((v) & (f)) == 0)

#define FlagsIsAllSet(v, f)             (((v) & (f)) == (f))
#define FlagsIsNotAllSet(v, f)          (((v) & (f)) != (f))
#define FlagsIsAllClr(v, f)             (((v) & (f)) == 0)
#define FlagsIsNotAllClr(v, f)          (((v) & (f)) != 0)

EXTERN const char* gp_header_path;
EXTERN const char* gp_lkr_path;
EXTERN const char* gp_lib_path;

FUNC(void) gp_init(void);

extern void     gp_fputl16(int16_t Data, FILE* Fp);
extern void     gp_fputl32(int32_t Data, FILE* Fp);
extern void     gp_fputzero(size_t Number, FILE* Fp);
extern void     gp_fputvar(const void* Data, size_t Number, FILE* Fp);
FUNC(uint16_t) gp_getu16(const uint8_t* Addr);
FUNC(int16_t)  gp_getl16(const uint8_t* Addr);
FUNC(uint32_t) gp_getu32(const uint8_t* Addr);
FUNC(int32_t)  gp_getl32(const uint8_t* Addr);
FUNC(void)     gp_putl16(uint8_t* Addr, uint16_t Data);
FUNC(void)     gp_putl32(uint8_t* Addr, uint32_t Data);

extern int32_t  gp_getb32(const uint8_t* Addr);
extern void     gp_putb32(uint8_t* Addr, uint32_t Data);

extern bool gp_num_range_is_overlapped(int Area_start, int Area_end, int Ref_start, int Ref_end);

extern unsigned gp_find_lowest_bit(uint64_t Bits);
extern unsigned gp_find_highest_bit(uint64_t Bits);

extern void  gp_date_string(char* Buffer, size_t Sizeof_buffer);

FUNC(void*) gp_malloc(size_t Size, const char* File, size_t Line, const char* Func);
FUNC(void*) gp_calloc(size_t Nmemb, size_t Size, const char* File, size_t Line, const char* Func);
FUNC(void*) gp_realloc(void* Mem, size_t Size, const char* File, size_t Line, const char* Func);
FUNC(char*) gp_strdup(const char *String, const char* File, size_t Line, const char* Func);
FUNC(char*) gp_strndup(const char *String, size_t Length, const char* File, size_t Line, const char* Func);

#define GP_Malloc(Size)                 gp_malloc(Size, __FILE__, __LINE__, __func__)
#define GP_Calloc(Nmemb, Size)          gp_calloc(Nmemb, Size, __FILE__, __LINE__, __func__)
#define GP_Realloc(Mem, Size)           gp_realloc(Mem, Size, __FILE__, __LINE__, __func__)
#define GP_Strdup(String)               gp_strdup(String, __FILE__, __LINE__, __func__)
#define GP_Strndup(String, Length)      gp_strndup(String, Length, __FILE__, __LINE__, __func__)

extern char* gp_strdup_lower_case(const char* Name);
extern char* gp_strdup_upper_case(const char* Name);
extern char* gp_strncpy(char* Dest, const char* Src, size_t Maxlen);
extern char* gp_arch_strncpy(char* Dest, const char* Src, size_t Maxlen);
extern char* gp_stptoupper(char* Dest, const char* Src, size_t Maxlen);

extern char* gp_str_from_Pstr(char* C_str, size_t C_max_size, const uint8_t *Pascal_str, size_t Pascal_max_size,
                              bool* Is_limited_length);

extern size_t gp_Pstr_from_str(uint8_t* Pascal_str, size_t Pascal_max_size, const char* C_str, bool* Is_limited_length);
extern size_t gp_strlen_Plimit(const char* C_str, size_t Pascal_max_size, bool* Is_limited_length);

extern size_t gp_align_text(char* Buffer, size_t Buffer_length, size_t Current_length, size_t Aligned_to_length);
extern size_t gp_exclamation(char* Buffer, size_t Buffer_length, size_t Current_length, const char* Format, ...);

extern char* gp_absolute_path(char* File_name);

extern void gp_exit_if_arg_an_option(const struct option* Options, int Opt_max_index, int Opt_index,
                                     const char* Opt_string, int Opt_char, const char* Command);
